@@ -20,12 +20,13 @@
  * OF THIS SOFTWARE.
  */
 
-#ifndef libevdev_INT_H
-#define libevdev_INT_H
+#ifndef LIBEVDEV_INT_H
+#define LIBEVDEV_INT_H
 
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <errno.h>
 #include "libevdev.h"
 
@@ -73,6 +74,7 @@ enum SyncState {
 
 struct libevdev {
 	int fd;
+	bool initialized;
 	char *name;
 	char *phys;
 	char *uniq;
