@@ -27,6 +27,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdarg.h>
+#include <stdbool.h>
 
 #include "libevdev.h"
 #include "libevdev-int.h"
@@ -109,19 +110,29 @@ log_msg(enum libevdev_log_priority priority,
 	va_end(args);
 }
 
+static void
+libevdev_reset(struct libevdev *dev)
+{
+	memset(dev, 0, sizeof(*dev));
+	dev->fd = -1;
+	dev->initialized = false;
+	dev->num_slots = -1;
+	dev->current_slot = -1;
+	dev->grabbed = LIBEVDEV_UNGRAB;
+	dev->sync_state = SYNC_NONE;
+	libevdev_enable_event_type(dev, EV_SYN);
+}
+
 LIBEVDEV_EXPORT struct libevdev*
 libevdev_new(void)
 {
 	struct libevdev *dev;
 
-	dev = calloc(1, sizeof(*dev));
+	dev = malloc(sizeof(*dev));
 	if (!dev)
 		return NULL;
-	dev->fd = -1;
-	dev->num_slots = -1;
-	dev->current_slot = -1;
-	dev->grabbed = LIBEVDEV_UNGRAB;
-	dev->sync_state = SYNC_NONE;
+
+	libevdev_reset(dev);
 
 	return dev;
 }
@@ -189,7 +200,7 @@ libevdev_get_log_priority(void)
 LIBEVDEV_EXPORT int
 libevdev_change_fd(struct libevdev *dev, int fd)
 {
-	if (dev->fd == -1) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -1;
 	}
@@ -204,10 +215,13 @@ libevdev_set_fd(struct libevdev* dev, int fd)
 	int i;
 	char buf[256];
 
-	if (dev->fd != -1) {
+	if (dev->initialized) {
 		log_bug("device already initialized.\n");
 		return -EBADF;
-	}
+	} else if (fd < 0)
+		return -EBADF;
+
+	libevdev_reset(dev);
 
 	rc = ioctl(fd, EVIOCGBIT(0, sizeof(dev->bits)), dev->bits);
 	if (rc < 0)
@@ -264,8 +278,12 @@ libevdev_set_fd(struct libevdev* dev, int fd)
 	if (rc < 0)
 		goto out;
 
+	/* Built on a kernel with props, running against a kernel without property
+	   support. This should not be a fatal case, we'll be missing properties but other
+	   than that everything is as expected.
+	 */
 	rc = ioctl(fd, EVIOCGPROP(sizeof(dev->props)), dev->props);
-	if (rc < 0)
+	if (rc < 0 && errno != EINVAL)
 		goto out;
 
 	rc = ioctl(fd, EVIOCGBIT(EV_REL, sizeof(dev->rel_bits)), dev->rel_bits);
@@ -351,7 +369,10 @@ libevdev_set_fd(struct libevdev* dev, int fd)
 	 * Same with the valuators, really, but they may not change.
 	 */
 
+	dev->initialized = true;
 out:
+	if (rc)
+		libevdev_reset(dev);
 	return rc ? -errno : 0;
 }
 
@@ -490,11 +511,14 @@ sync_mt_state(struct libevdev *dev, int create_events)
 {
 	int rc;
 	int i;
+	int ioctl_success = 0;
 	struct mt_state {
 		int code;
 		int val[MAX_SLOTS];
 	} mt_state[ABS_MT_CNT];
 
+	memset(&mt_state, 0, sizeof(mt_state));
+
 	for (i = ABS_MT_MIN; i <= ABS_MT_MAX; i++) {
 		int idx;
 		if (i == ABS_MT_SLOT)
@@ -506,8 +530,15 @@ sync_mt_state(struct libevdev *dev, int create_events)
 		idx = i - ABS_MT_MIN;
 		mt_state[idx].code = i;
 		rc = ioctl(dev->fd, EVIOCGMTSLOTS(sizeof(struct mt_state)), &mt_state[idx]);
-		if (rc < 0)
-			goto out;
+		if (rc < 0) {
+			/* if the first ioctl fails with -EINVAL, chances are the kernel
+			   doesn't support the ioctl. Simply continue */
+			if (errno == -EINVAL && !ioctl_success) {
+				rc = 0;
+			} else /* if the second, ... ioctl fails, really fail */
+				goto out;
+		} else if (ioctl_success == 0)
+			ioctl_success = 1;
 	}
 
 	for (i = 0; i < dev->num_slots; i++) {
@@ -731,10 +762,11 @@ libevdev_next_event(struct libevdev *dev, unsigned int flags, struct input_event
 {
 	int rc = LIBEVDEV_READ_STATUS_SUCCESS;
 
-	if (dev->fd < 0) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -EBADF;
-	}
+	} else if (dev->fd < 0)
+		return -EBADF;
 
 	if (!(flags & (LIBEVDEV_READ_FLAG_NORMAL|LIBEVDEV_READ_FLAG_SYNC|LIBEVDEV_READ_FLAG_FORCE_SYNC))) {
 		log_bug("invalid flags %#x\n.\n", flags);
@@ -821,10 +853,11 @@ libevdev_has_event_pending(struct libevdev *dev)
 	struct pollfd fds = { dev->fd, POLLIN, 0 };
 	int rc;
 
-	if (dev->fd < 0) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -EBADF;
-	}
+	} else if (dev->fd < 0)
+		return -EBADF;
 
 	if (queue_num_elements(dev) != 0)
 		return 1;
@@ -912,7 +945,7 @@ libevdev_enable_property(struct libevdev *dev, unsigned int prop)
 LIBEVDEV_EXPORT int
 libevdev_has_event_type(const struct libevdev *dev, unsigned int type)
 {
-	return (type <= EV_MAX) && bit_is_set(dev->bits, type);
+	return type == EV_SYN ||(type <= EV_MAX && bit_is_set(dev->bits, type));
 }
 
 LIBEVDEV_EXPORT int
@@ -1106,12 +1139,18 @@ libevdev_set_abs_info(struct libevdev *dev, unsigned int code, const struct inpu
 LIBEVDEV_EXPORT int
 libevdev_enable_event_type(struct libevdev *dev, unsigned int type)
 {
+	int max;
+
 	if (type > EV_MAX)
 		return -1;
 
 	if (libevdev_has_event_type(dev, type))
 		return 0;
 
+	max = libevdev_event_type_get_max(type);
+	if (max == -1)
+		return -1;
+
 	set_bit(dev->bits, type);
 
 	if (type == EV_REP) {
@@ -1125,9 +1164,15 @@ libevdev_enable_event_type(struct libevdev *dev, unsigned int type)
 LIBEVDEV_EXPORT int
 libevdev_disable_event_type(struct libevdev *dev, unsigned int type)
 {
+	int max;
+
 	if (type > EV_MAX || type == EV_SYN)
 		return -1;
 
+	max = libevdev_event_type_get_max(type);
+	if (max == -1)
+		return -1;
+
 	clear_bit(dev->bits, type);
 
 	return 0;
@@ -1159,7 +1204,7 @@ libevdev_enable_event_code(struct libevdev *dev, unsigned int type,
 
 	max = type_to_mask(dev, type, &mask);
 
-	if (code > max)
+	if (code > max || (int)max == -1)
 		return -1;
 
 	set_bit(mask, code);
@@ -1181,12 +1226,12 @@ libevdev_disable_event_code(struct libevdev *dev, unsigned int type, unsigned in
 	unsigned int max;
 	unsigned long *mask = NULL;
 
-	if (type > EV_MAX)
+	if (type > EV_MAX || type == EV_SYN)
 		return -1;
 
 	max = type_to_mask(dev, type, &mask);
 
-	if (code > max)
+	if (code > max || (int)max == -1)
 		return -1;
 
 	clear_bit(mask, code);
@@ -1205,10 +1250,11 @@ libevdev_kernel_set_abs_info(struct libevdev *dev, unsigned int code, const stru
 {
 	int rc;
 
-	if (dev->fd < 0) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -EBADF;
-	}
+	} else if (dev->fd < 0)
+		return -EBADF;
 
 	if (code > ABS_MAX)
 		return -EINVAL;
@@ -1227,10 +1273,11 @@ libevdev_grab(struct libevdev *dev, enum libevdev_grab_mode grab)
 {
 	int rc = 0;
 
-	if (dev->fd < 0) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -EBADF;
-	}
+	} else if (dev->fd < 0)
+		return -EBADF;
 
 	if (grab != LIBEVDEV_GRAB && grab != LIBEVDEV_UNGRAB) {
 		log_bug("invalid grab parameter %#x\n", grab);
@@ -1372,10 +1419,11 @@ libevdev_kernel_set_led_values(struct libevdev *dev, ...)
 	int rc = 0;
 	size_t nleds = 0;
 
-	if (dev->fd < 0) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -EBADF;
-	}
+	} else if (dev->fd < 0)
+		return -EBADF;
 
 	memset(ev, 0, sizeof(ev));
 
@@ -1427,10 +1475,11 @@ libevdev_kernel_set_led_values(struct libevdev *dev, ...)
 LIBEVDEV_EXPORT int
 libevdev_set_clock_id(struct libevdev *dev, int clockid)
 {
-	if (dev->fd < 0) {
+	if (!dev->initialized) {
 		log_bug("device not initialized. call libevdev_set_fd() first\n");
 		return -EBADF;
-	}
+	} else if (dev->fd < 0)
+		return -EBADF;
 
 	return ioctl(dev->fd, EVIOCSCLOCKID, &clockid) ? -errno : 0;
 }
