@@ -135,6 +135,12 @@ extern "C" {
  * A more complete example is available with the libevdev-events tool here:
  * http://cgit.freedesktop.org/libevdev/tree/tools/libevdev-events.c
  *
+ * Backwards compatibility with older kernel
+ * =========================================
+ * libevdev attempts to build and run correctly on a number of kernel versions.
+ * If features required are not available, libevdev attempts to work around them
+ * in the most reasonable way. For more details see \ref backwardscompatibility.
+ *
  * libevdev internal test suite
  * ============================
  *
@@ -189,6 +195,56 @@ extern "C" {
  * https://bugs.freedesktop.org/enter_bug.cgi?product=libevdev
  */
 
+/**
+ * @page backwardscompatibility Compatibility and Behavior across kernel versions
+ *
+ * This page describes libevdev's behavior when the build-time kernel and the
+ * run-time kernel differ in their feature set.
+ *
+ * With the exception of event names, libevdev defines features that may be
+ * missing on older kernels and building on such kernels will not disable
+ * features. Running libevdev on a kernel that is missing some feature will
+ * change libevdev's behavior. In most cases, the new behavior should be
+ * obvious, but it is spelled out below in detail.
+ *
+ * Minimum requirements
+ * ====================
+ * libevdev requires a 2.6.36 kernel as minimum. Specifically, it requires
+ * ABS_MT_SLOT and the matching behavior.
+ *
+ * Event and input property names
+ * ==============================
+ * Event names and input property names are defined at build-time by the
+ * linux/input.h shipped with libevedv.
+ * The list of event names is compiled at build-time and thus any events not defined
+ * at build time will not resolve. Specifically,
+ * libevdev_event_code_get_name(type, code) for an undefined type or code will
+ * always return NULL. Likewise, libevdev_property_get_name() will return NULL
+ * for properties undefined on the build system.
+ *
+ * Input properties
+ * ================
+ * If the kernel does not support input properties, specifically the
+ * EVIOCGPROPS ioclt, libevdev does not expose input properties to the caller.
+ * Specifically, libevdev_has_property() will always return 0 unless the
+ * property has been manually set with libevdev_enable_property().
+ *
+ * This also applies to the libevdev-uinput code. If uinput does not honor
+ * UI_SET_PROPBIT, libevdev will continue without setting the properties on
+ * the device.
+ *
+ * MT slot behavior
+ * =================
+ * If the kernel does not support the EVIOCGMTSLOTS ioctl, libevdev continues
+ * as normal and assumes all values in all slots to be 0.
+ *
+ * SYN_DROPPED behavior
+ * ====================
+ * A kernel without SYN_DROPPED, won't send the event. libevdev_next_event()
+ * will never require the switch to sync mode.
+ *
+ */
+
 /**
  * @defgroup init Initialization and setup
  *
@@ -478,6 +534,10 @@ int libevdev_set_fd(struct libevdev* dev, int fd);
  * No check is made that new fd points to the same device. If the device has
  * changed, libevdev's behavior is undefined.
  *
+ * libevdev does not sync itself after changing the fd and keeps the current
+ * device state. Use libevdev_next_event with the LIBEVDEV_FORCE_SYNC flag to
+ * force a re-sync.
+ *
  * The fd may be open in O_RDONLY or O_RDWR.
  *
  * It is an error to call this function before calling libevdev_set_fd().
@@ -1399,6 +1459,85 @@ const char* libevdev_property_get_name(unsigned int prop);
  */
 int libevdev_event_type_get_max(unsigned int type);
 
+/**
+ * @ingroup misc
+ *
+ * Look up an event-type by its name. Event-types start with "EV_" followed by
+ * the name (eg., "EV_ABS"). The "EV_" prefix must be included in the name. It
+ * returns the constant assigned to the event-type or -1 if not found.
+ *
+ * @param name A non-NULL string describing an input-event type ("EV_KEY",
+ * "EV_ABS", ...), zero-terminated.
+ *
+ * @return The given type constant for the passed name or -1 if not found.
+ *
+ * @note EV_MAX is also recognized.
+ */
+int libevdev_event_type_from_name(const char *name);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event-type by its name. Event-types start with "EV_" followed by
+ * the name (eg., "EV_ABS"). The "EV_" prefix must be included in the name. It
+ * returns the constant assigned to the event-type or -1 if not found.
+ *
+ * @param name A non-NULL string describing an input-event type ("EV_KEY",
+ * "EV_ABS", ...).
+ * @param len The length of the passed string excluding any terminating 0
+ * character.
+ *
+ * @return The given type constant for the passed name or -1 if not found.
+ *
+ * @note EV_MAX is also recognized.
+ */
+int libevdev_event_type_from_name_n(const char *name, size_t len);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event-code by its type and name. Event-codes start with a fixed
+ * prefix followed by their name (eg., "ABS_X"). The prefix must be included in
+ * the name. It returns the constant assigned to the event-code or -1 if not
+ * found.
+ *
+ * You have to pass the event-type where to look for the name. For instance, to
+ * resolve "ABS_X" you need to pass EV_ABS as type and "ABS_X" as string.
+ * Supported event-codes are codes starting with SYN_, KEY_, BTN_, REL_, ABS_,
+ * MSC_, SND_, SW_, LED_, REP_, FF_.
+ *
+ * @param type The event type (EV_* constant) where to look for the name.
+ * @param name A non-NULL string describing an input-event code ("KEY_A",
+ * "ABS_X", "BTN_Y", ...), zero-terminated.
+ *
+ * @return The given code constant for the passed name or -1 if not found.
+ */
+int libevdev_event_code_from_name(unsigned int type, const char *name);
+
+/**
+ * @ingroup misc
+ *
+ * Look up an event-code by its type and name. Event-codes start with a fixed
+ * prefix followed by their name (eg., "ABS_X"). The prefix must be included in
+ * the name. It returns the constant assigned to the event-code or -1 if not
+ * found.
+ *
+ * You have to pass the event-type where to look for the name. For instance, to
+ * resolve "ABS_X" you need to pass EV_ABS as type and "ABS_X" as string.
+ * Supported event-codes are codes starting with SYN_, KEY_, BTN_, REL_, ABS_,
+ * MSC_, SND_, SW_, LED_, REP_, FF_.
+ *
+ * @param type The event type (EV_* constant) where to look for the name.
+ * @param name A non-NULL string describing an input-event code ("KEY_A",
+ * "ABS_X", "BTN_Y", ...).
+ * @param len The length of the passed string excluding any terminating 0
+ * character.
+ *
+ * @return The given code constant for the passed name or -1 if not found.
+ */
+int libevdev_event_code_from_name_n(unsigned int type, const char *name,
+				    size_t len);
+
 /**
  * @ingroup bits
  *
@@ -1438,7 +1577,7 @@ void libevdev_set_log_handler(struct libevdev *dev, libevdev_log_func_t logfunc)
 int libevdev_get_event_type_max(unsigned int type) LIBEVDEV_DEPRECATED;
 
 /** replacement: libevdev_property_get_name */
-const char* libevdev_get_property_name(unsigned int prop);
+const char* libevdev_get_property_name(unsigned int prop) LIBEVDEV_DEPRECATED;
 
 /** replacement: libevdev_event_type_get_name */
 const char * libevdev_get_event_type_name(unsigned int type) LIBEVDEV_DEPRECATED;
@@ -1446,10 +1585,10 @@ const char * libevdev_get_event_type_name(unsigned int type) LIBEVDEV_DEPRECATED
 const char * libevdev_get_event_code_name(unsigned int type, unsigned int code) LIBEVDEV_DEPRECATED;
 
 /** replacement: libevdev_event_is_type */
-int libevdev_is_event_type(const struct input_event *ev, unsigned int type);
+int libevdev_is_event_type(const struct input_event *ev, unsigned int type) LIBEVDEV_DEPRECATED;
 
 /** replacement: libevdev_event_is_code */
-int libevdev_is_event_code(const struct input_event *ev, unsigned int type, unsigned int code);
+int libevdev_is_event_code(const struct input_event *ev, unsigned int type, unsigned int code) LIBEVDEV_DEPRECATED;
 /**************************************/
 
 #ifdef __cplusplus
